@@ -26,16 +26,19 @@
 #
 #-------------------------------------------------------------------------
 
-$cgextensions = cms_join_path($gCms->config['root_path'],'modules',
-	'CGExtensions','CGExtensions.module.php');
-if( !is_readable( $cgextensions ) )
+$cgpaymentgatewaybase = cms_join_path($gCms->config['root_path'],'modules',
+	'CGPaymentGatewayBase','CGPaymentGatewayBase.module.php');
+if( !is_readable( $cgpaymentgatewaybase ) )
 {
-	echo '<h1><font color="red">ERROR: The CGExtensions module could not be found.</font></h1>';
+	echo '<h1><font color="red">ERROR: The CGPaymentGatewayBase module could not be found.  Please install it</font></h1>';
 	return;
 }
-require_once($cgextensions);
+if( !class_exists('CGPaymentGatewayBase') )
+{
+  require_once($cgpaymentgatewaybase);
+}
 
-class EMoneyGateway extends CGExtensions
+class EMoneyGateway extends CGPaymentGatewayBase
 {
 
   var $_weightunits;
@@ -55,7 +58,7 @@ class EMoneyGateway extends CGExtensions
   function EMoneyGateway()
   {
     parent::CMSModule();
-    $this->reset();
+    $this->Reset();
   }
 
   /*---------------------------------------------------------
@@ -80,7 +83,7 @@ class EMoneyGateway extends CGExtensions
    ---------------------------------------------------------*/
   function GetVersion()
   {
-    return '1.0.2';
+    return '2.1';
   }
 
 
@@ -170,7 +173,8 @@ class EMoneyGateway extends CGExtensions
    ---------------------------------------------------------*/
   function GetDependencies()
   {
-    return array('CGExtensions'=>'1.9','Orders'=>'1.1');
+    return array('CGPaymentGatewayBase'=>'1.0.3',
+		 'CGExtensions'=>'1.17.3');
   }
 
 
@@ -179,7 +183,7 @@ class EMoneyGateway extends CGExtensions
    ---------------------------------------------------------*/
   function MinimumCMSVersion()
   {
-    return "1.3";
+    return "1.6.5";
   }
 
 
@@ -191,15 +195,7 @@ class EMoneyGateway extends CGExtensions
     $this->RegisterModulePlugin();
     $this->RestrictUnknownParams();
 
-    $this->SetParameterType('process', CLEAN_STRING);		
-    $this->SetParameterType('order_id', CLEAN_INT);
-    $this->SetParameterType('mycustom', CLEAN_STRING);
-	$this->SetParameterType('emgw_custom_field', CLEAN_STRING);
-	$this->SetParameterType('card_number', CLEAN_STRING);
-	$this->SetParameterType('cvv', CLEAN_STRING);
-	$this->SetParameterType('exp_month', CLEAN_INT);
-	$this->SetParameterType('exp_year', CLEAN_INT);
-	$this->SetParameterType('submit', CLEAN_STRING);
+	$this->SetParameterType('key', CLEAN_STRING);
 	
   }
 
@@ -239,15 +235,186 @@ class EMoneyGateway extends CGExtensions
     return true;
   }
 
-  function GetForm($returnid)
+  /*---------------------------------------------------------
+   New Methods from CGPaymentGatewayBase
+   ---------------------------------------------------------*/
+  
+/*---------------------------------------------------------
+ RequiresCreditCardInfo()
+ ---------------------------------------------------------*/
+public function RequiresCreditCardInfo()
+{
+  return true;
+}
+
+
+/*---------------------------------------------------------
+ RequiresSSL()
+ ---------------------------------------------------------*/
+public function RequiresSSL()
+{
+  return true;
+}
+
+
+/*---------------------------------------------------------
+ RequiresShippingInfo()
+ ---------------------------------------------------------*/
+public function RequiresShippingInfo()
+{
+  return false;
+}
+
+
+/*---------------------------------------------------------
+ RequiresBillingInfo()
+ ---------------------------------------------------------*/
+public function RequiresBillingInfo()
+{
+  return false;
+}
+
+/*---------------------------------------------------------
+  SaveState()
+  ---------------------------------------------------------*/
+ public function SaveState($encrypt = false)
+ {
+   if( $encrypt )
+     {
+	$key = str_shuffle(md5(session_id().time()));
+	$raw = serialize($this->_data);
+	$enc = $this->encrypt($key,$raw);
+	$_SESSION['emoneygate'] = base64_encode($enc);
+	return $key;
+     }
+   else
+     {
+	$raw = serialize($this->_data);
+	$_SESSION['emoneygate'] = base64_encode($raw);
+     }
+ }
+
+
+ /*---------------------------------------------------------
+  RestoreState()
+  ---------------------------------------------------------*/
+ public function RestoreState($key = '')
+ {
+   if( !isset($_SESSION['emoneygate']) ) return FALSE;
+   if( empty($key) )
+     {
+	$raw = base64_decode($_SESSION['emoneygate']);
+     }
+   else
+     {
+	$enc = base64_decode($_SESSION['emoneygate']);
+	$raw = $this->decrypt($key,$enc);
+	if( $raw === FALSE ) return FALSE;
+     }
+   $this->_data = unserialize($raw);
+   unset($_SESSION['emoneygate']);
+   return TRUE;
+ }
+
+
+ /*---------------------------------------------------------
+  SetCreditCardInfo()
+  ---------------------------------------------------------*/
+ public function SetCreditCardInfo($ccnumber,$ccexp,$ccvcode)
+ {
+   $_ccinfo = array();
+   $_ccinfo['x_card_num'] = preg_replace('([^0-9])','',$ccnumber);
+   $_ccinfo['x_exp_date'] = date('m',$ccexp).'/'.date('Y',$ccexp);
+   if( !empty($ccvcode) )
+     {
+	$_ccinfo['x_card_code'] = preg_replace('([^0-9])','',$ccvcode);
+     }
+   debug_to_log('-- ccinfo --');
+   debug_to_log($_ccinfo);
+   $this->_data['ccinfo'] = $_ccinfo;
+ }
+
+
+ /*---------------------------------------------------------
+  SetWeightUnits()
+  ---------------------------------------------------------*/
+ public function SetWeightUnits($units = 'lbs')
+ {
+   $this->_data['weightunits'] = $units;
+ }
+
+
+ /*---------------------------------------------------------
+  SetDestination()
+  ---------------------------------------------------------*/
+ public function SetDestination($url)
+ {
+   $this->_data['destination'] = $url;
+ }
+   
+ /*---------------------------------------------------------
+  ProcessTransaction()
+  ---------------------------------------------------------*/
+ public function ProcessTransaction()
+ {
+   $key = $this->SaveState(true);
+   $params = array();
+   $params['key'] = $key;
+   return $this->DoAction('process_payment','cntnt01',$params);
+ }
+
+ /*---------------------------------------------------------
+  CheckInfo()
+  ---------------------------------------------------------*/
+ public function CheckInfo()
+ {
+   // TODO: fill this in.
+   return TRUE;
+ }
+
+
+  /*---------------------------------------------------------
+   End New Methods from CGPaymentGatewayBase
+   ---------------------------------------------------------*/
+
+/*---------------------------------------------------------
+ GetForm()
+ ---------------------------------------------------------*/
+public function GetForm($returnid)
+{
+  if( !$this->CheckInfo() ) return FALSE;
+/*  $loginid = $this->GetPreference('loginid');
+  $transactionkey = $this->GetPreference('transactionkey');
+  if( empty($loginid) || empty($transactionkey) )
+    {
+	return $this->DisplayErrorMessage($this->GetName().': '.$this->Lang('error_configuration'));
+    }
+*/  
+  $key = $this->SaveState(true);
+
+  global $gCms;
+  $smarty =& $gCms->GetSmarty();
+  $smarty->assign('formstart',
+		    $this->CGCreateFormStart('cntnt01','process',$returnid,
+					     array('key'=>$key)));
+  $smarty->assign('formend',$this->CreateFormEnd());
+		    
+					     
+  // and process the template.
+  return $this->ProcessTemplate('newform.tpl');
+}
+
+
+
+  function GetFormOld($returnid)
 	{
 	// force https?
 	$id = 'cntnt07';
 		
+	$key = $this->SaveState(true);
+
 	$this->smarty->assign('form_start',	$this->CreateFrontendFormStart($id,$returnid,'process','post',
-		'',false,'',array('order_id'=>$this->_orderid,'emgw_custom_field'=>md5('order'.$this->_orderid)
-					    )));
-	$_SESSION['emgw_custom_field'] = md5('order'.$this->_orderid);
+		'',false,'',array('key'=>$key)));
 
 	$this->smarty->assign('prompt_card',$this->Lang('prompt_card'));
 	$this->smarty->assign('prompt_expiration',$this->Lang('prompt_expiration'));
@@ -270,182 +437,273 @@ class EMoneyGateway extends CGExtensions
 		}
 	
 	$this->smarty->assign('input_expiration', $this->CreateInputDropdown($id, 'exp_month', $mon). $this->CreateInputDropdown($id, 'exp_year', $year));
-    if( !empty($this->_successurl) )
-      {
-		$_SESSION['emgw_success_url'] = $this->_successurl;
-      }
-    if( !empty($this->_cancelurl) )
-      {
-		$_SESSION['emgw_cancel_url'] = $this->_cancelurl;
-      }
 	return $this->ProcessTemplate('form.tpl');
 	}
 
-/*
-  function GetForm($returnid)
-  {
-    if( empty($this->_orderid) || count($this->_items) == 0 )
-      {
-		return '';
-      }
-    $successpage = (empty($this->_successpage))?$returnid:$this->_successpage;
-    $cancelpage = (empty($this->_cancelpage))?$returnid:$this->_cancelpage;
-    
-    require_once('paypal.class.php');
-    $pp = new paypal_class;
-
-    $pp->paypal_url = $this->GetPreference('paypal_url','https://www.sandbox.paypal.com/cgi-bin/webscr');
-    $pp->add_field('upload', '1'); //3rd party cart
-    $pp->add_field('rm','2'); //Use POST for returns
-    $pp->add_field('cmd','_cart'); //3rd party cart
-    if( !empty($this->_weightunits) )
-      {
-	$pp->add_field('weight_unit',$this->_weightunits);
-      }
-    if( !empty($this->_currencycode) )
-      {
-	$pp->add_field('currency_code',$this->_currencycode);
-      }
-    $pp->add_field('business',$this->GetPreference('business_email',''));
-    $pp->add_field('order_id',$this->_orderid);
-    $_SESSION['ppgw_custom_field'] = md5('orders'.$this->_orderid);
-    $pp->add_field('custom',$_SESSION['ppgw_custom_field']);
-    $pp->add_field('return',$this->CreateLink($id,'return_success',$successpage,'',array('order_id'=>$this->_orderid,
-											 'mycustom'=>$_SESSION['ppgw_custom_field']),'',true));
-    $pp->add_field('cancel_return',
- 		   $this->CreateLink($id,'return_cancel',$cancelpage,'',array('order_id'=>$this->_orderid,
-									      'mycustom'=>$_SESSION['ppgw_custom_field']),'',true));
-    $pp->add_field('no_note',1);
-
-    // todo... handle a notify url for ipn
-
-    $count = 1;
-    foreach( $this->_items as $name => $data )
-      {
-	$pp->add_field('amount_'.$count, $data['amount']);
-	$pp->add_field('item_name_'.$count, $name );
-	$pp->add_field('item_number_'.$count, $data['number']);
-	$pp->add_field('quantity_'.$count, $data['quantity']);
-	$count++;
-      }
-    
-    // hack for shipping
-    if( !empty($this->_ordershipping) && $this->_ordershipping > 0 )
-      {
-	$pp->add_field('amount_'.$count, $this->_ordershipping );
-	$pp->add_field('item_name_'.$count, 'Shipping');
-      }
-
-    // taxes
-    if( !empty($this->_ordertax) )
-      {
-	$pp->add_field('tax_cart',$this->_ordertax);
-      }
-
-    // set a couple of session variables for the return urls
-    if( !empty($this->_successurl) )
-      {
-	$_SESSION['ppgw_success_url'] = $this->_successurl;
-      }
-    if( !empty($this->_cancelurl) )
-      {
-	$_SESSION['ppgw_cancel_url'] = $this->_cancelurl;
-      }
-    @ob_start();
-    $pp->submit_paypal_post(false,$this->Lang('checkout_message').' '.
-			    '<input type="submit" value="'.$this->Lang('checkout_button_message').'">');
-    $result = @ob_get_contents();
-    @ob_end_clean();
-    return $result;
-  }
-*/
-
-  function reset()
-  {
-    $this->_weightunits = 'lbs';
-    $this->_currencycode = 'USD';
-    $this->_successurl = '';
-    $this->_cancelurl = '';
-    $this->_successpage = '';
-    $this->_cancelpage = '';
-    $this->_items = array();
-    $this->_ordertax = '';
-    $this->_ordershipping = '';
-    $this->_orderid = '';
-  }
-
-
   function SetCurrencyCode($code = 'USD')
   {
-    $this->_currencycode = strtoupper($code);
-  }
-
-
-  function SetWeightUnits($units = 'lbs')
-  {
-    $this->_weightunits = $units;
+    $this->_data['currencycode'] = strtoupper($code);
   }
 
 
   function SetSuccessURL($url)
   {
-    $this->_successurl = $url;
+    $this->_data['successurl'] = $url;
   }
 
 
   function SetSuccessPage($page)
   {
-    $this->_successpage = $page;
+    $this->_data['successpage'] = $page;
   }
 
 
   function SetCancelURL($url)
   {
-    $this->_cancelurl = $url;
+    $this->_data['cancelurl'] = $url;
   }
 
 
   function SetCancelPage($page)
   {
-    $this->_cancelpage = $page;
+    $this->_data['cancelpage'] = $page;
   }
 
-
-  function AddItem($name,$number,$quantity,$weight,$amount,$tax = '')
+  /*---------------------------------------------------------
+   AddItem()
+   ---------------------------------------------------------*/
+  public function AddItem($name,$number,$quantity,$weight,$amount,$tax = '')
   {
+    $name = strip_tags($name);
+    $name = preg_replace('/\&.*\;/','',$name);
+    $name = trim($name);
+
+    if( !isset($this->_data['items']) )
+      {
+	$this->_data['items'] = array();
+      }
+
     if( !isset($this->_items[$name]) )
       {
-	$this->_items[$name] = array('name'=>$name,
+	$this->_data['items'][$name] = array('name'=>$name,
 				     'number'=>$number,
 				     'quantity'=>$quantity,
 				     'weight'=>$weight,
 				     'amount'=>$amount);
 	if(!empty($tax))
 	  {
-	    $this->_items[$name]['tax'] = $tax;
+	    $this->_data['items'][$name]['tax'] = $tax;
 	  }
       }
   }
 
-
-  function SetOrderTaxAmount($tax)
+  /*---------------------------------------------------------
+   SetOrderDescription()
+   ---------------------------------------------------------*/
+  public function SetOrderDescription($txt)
   {
-    $this->_ordertax = $tax;
+    $this->_data['description'] = substr($txt,0,255);
   }
 
 
-  function SetOrderShipping($shipping)
+  /*---------------------------------------------------------
+   SetOrderTaxAmount()
+   ---------------------------------------------------------*/
+  public function SetOrderTaxAmount($tax)
   {
-    $this->_ordershipping = $shipping;
+    $this->_data['ordertax'] = $tax;
   }
 
 
-  function SetOrderID($orderid)
+  /*---------------------------------------------------------
+   SetOrderShipping()
+   ---------------------------------------------------------*/
+  public function SetOrderShipping($shipping)
   {
-    $this->_orderid = $orderid;
+    $this->_data['ordershipping'] = $shipping;
   }
 
 
+
+  /*---------------------------------------------------------
+   SetInvoice()
+   ---------------------------------------------------------*/
+  public function SetInvoice($invoice)
+  {
+    $len = min(strlen($invoice),20)*-1;
+    $this->_data['invoice'] = substr($invoice,$len);
+  }
+
+
+  /*---------------------------------------------------------
+   SetOrderID()
+   ---------------------------------------------------------*/
+  public function SetOrderID($orderid)
+  {
+    $this->_data['orderid'] = $orderid;
+  }
+
+
+  /*---------------------------------------------------------
+   GetOrderID()
+   ---------------------------------------------------------*/
+  public function GetOrderID()
+  {
+    if( isset($this->_data['orderid']) )
+      return $this->_data['orderid'];
+  }
+
+
+  /*---------------------------------------------------------
+   SetPaymentid()
+   ---------------------------------------------------------*/
+  public function SetPaymentid($paymentid)
+  {
+    $this->_data['paymentid'] = $paymentid;
+  }
+
+
+  /*---------------------------------------------------------
+   GetPaymentID()
+   ---------------------------------------------------------*/
+  public function GetPaymentID()
+  {
+    if( isset($this->_data['paymentid']) )
+      return $this->_data['paymentid'];
+  }
+
+
+  /*---------------------------------------------------------
+   SetTransactionId()
+   ---------------------------------------------------------*/
+  public function SetTransactionId($tid)
+  {
+    $this->_data['transaction_id'] = $tid;
+  }
+
+
+  /*---------------------------------------------------------
+   SetTransactionStatus()
+   ---------------------------------------------------------*/
+  public function SetTransactionStatus($status)
+  {
+    $this->_data['status'] = $status;
+  }
+
+
+  /*---------------------------------------------------------
+   SetTransactionAmount()
+   ---------------------------------------------------------*/
+  public function SetTransactionAmount($tid)
+  {
+    $this->_data['amount'] = $tid;
+  }
+
+
+  /*---------------------------------------------------------
+   SetError()
+   ---------------------------------------------------------*/
+  public function SetError($message)
+  {
+    $this->_data['status'] = PAYMENT_STATUS_ERROR;
+    $this->_data['error_message'] = $message;
+  }
+
+
+  /*---------------------------------------------------------
+   SetCancelled()
+   ---------------------------------------------------------*/
+  public function SetCancelled($message)
+  {
+    $this->_data['status'] = PAYMENT_STATUS_CANCELLED;
+  }
+
+
+  /*---------------------------------------------------------
+   Reset()
+   ---------------------------------------------------------*/
+  public function Reset()
+  {
+    $this->_data = array();
+  }
+
+
+  /*---------------------------------------------------------
+   SetDeclined()
+   ---------------------------------------------------------*/
+  public function SetDeclined($message='')
+  {
+    $this->_data['status'] = PAYMENT_STATUS_DECLINED;
+	if ($message == '')
+		{
+    	$this->_data['error_message'] = $this->Lang('error_payment_declined');
+		}
+	else
+		{
+		$this->_data['error_message'] = $message;	
+		}
+  }
+
+
+  /*---------------------------------------------------------
+   GetTransactionStatus()
+   ---------------------------------------------------------*/
+  public function GetTransactionStatus()
+  {
+    if( isset($this->_data['status']) )
+      return $this->_data['status'];
+    return FALSE;
+  }
+
+
+  /*---------------------------------------------------------
+   GetTransactionId()
+   ---------------------------------------------------------*/
+  public function GetTransactionId()
+  {
+    if( isset($this->_data['transaction_id']) )
+      return $this->_data['transaction_id'];
+    return FALSE;
+  }
+
+
+  /*---------------------------------------------------------
+   GetTransactionAmount()
+   ---------------------------------------------------------*/
+  public function GetTransactionAmount()
+  {
+    if( isset($this->_data['amount']) )
+      return $this->_data['amount'];
+    return FALSE;
+  }
+
+
+  /*---------------------------------------------------------
+   GetMessage()
+   ---------------------------------------------------------*/
+  public function GetMessage()
+  {
+    if( isset($this->_data['error_message']) )
+      return $this->_data['error_message'];
+    return FALSE;
+  }
+
+
+  /*---------------------------------------------------------
+   FinishTransaction()
+   ---------------------------------------------------------*/
+  public function FinishTransaction()
+  {
+    if( !isset($this->_data['destination']) ) return FALSE;
+    $key = $this->SaveState();
+    $url = $this->_data['destination'];
+    $url = html_entity_decode($url);
+    $url .= "&cntnt01datakey=$key";
+    redirect($url);
+  }
+
+
+
   function SetReturnStatus($status)
   {
     if( !isset($_SESSION['gateway_return_params']) )
